#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    : 2021/1/7 23:44
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : htmlParse.py
# @Desc    ：
# @Software: PyCharm

# 网页自动解析内容需要过滤的文本
filterValueArray = [
"忘记密码",
"注册用户",
"我要提问",
"清空",
"反馈/投诉"

]